/*=========================================================================

  Program:   wxMaracas
  Module:    $RCSfile: manualContourContainer.h,v $
  Language:  C++
  Date:      $Date: 2009/05/19 11:17:26 $
  Version:   $Revision: 1.3 $

  Copyright: (c) 2002, 2003
  License:
  
     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/

#ifndef MANUAL_CONTOUR_CONTAINER_H
#define MANUAL_CONTOUR_CONTAINER_H


#include <vector>
#include "manualContourControler.h"
#include "manualContourModel.h"
#include "manualViewContour.h"

class manualContourContainer  
{
public:
	
	manualContourContainer();
	virtual ~manualContourContainer();

	manualContourControler* getWallController();
	manualContourControler* getLumenController();
	manualContourControler* getCalcController(int i);
	manualContourControler* getHypoController(int i);
	void createWallContour(wxVtk2DBaseView	*_imageviewer2D_1);
	void createLumenContour(wxVtk2DBaseView	*_imageviewer2D_1);

	int	addCalcificationContour(wxVtk2DBaseView	*_imageviewer2D_1);
	int	addHypodenseContour(wxVtk2DBaseView	*_imageviewer2D_1);

	void setWallVisibility(bool visible);
	
	void setLumenVisibility(bool visible);
	void setCalcVisibility(bool visible);
	void setHypoVisibility(bool visible);
	void setWallActive();
	void setLumenActive();
	void setCalcActive();
	void setHypoActive();
	void setCalcContourActive(int i);
	void setHypoContourActive(int i);
	int  getNumberOfCalcContours();
	int  getNumberOfHypoContours();
	void inactivate();

	int getNumberOfWallContourPoints();
	int getNumberOfLumenContourPoints();
	int getNumberOfCalcContourPoints(int i);
	int getNumberOfHypoContourPoints(int i);

	double* getXVectorWallPoints();
	double* getXVectorLumenPoints();
	double* getXVectorCalcPoints(int i);
	double* getXVectorHypoPoints(int i);

	double* getYVectorWallPoints();
	double* getYVectorLumenPoints();
	double* getYVectorCalcPoints(int i);
	double* getYVectorHypoPoints(int i);
	void clear();
	void clearWall();
	void clearLumen();
	void clearCalc();
	void clearHypo();
	void refreshWall(wxVtk2DBaseView	*_imageviewer2D_1);
	void refreshLumen(wxVtk2DBaseView	*_imageviewer2D_1);
	void refreshCalc(int i, wxVtk2DBaseView	*_imageviewer2D_1);
	void refreshHypo(int i, wxVtk2DBaseView	*_imageviewer2D_1);
	
	void restartWallContour(wxVtk2DBaseView	*_imageviewer2D_1);
	void restartLumenContour(wxVtk2DBaseView	*_imageviewer2D_1);
	void restartCalcContours();
	void restartHypoContours();

private:
	//Manual Contour Wall
	
	manualContourControler *_manContourControlWall;
	manualContourModel		*_mContourModelWall;
	manualViewContour		*_mViewContourWall;


	//Manual Contour Lumen
	manualContourControler	*_manContourControlLumen;
	manualContourModel		*_mContourModelLumen;
	manualViewContour			*_mViewContourLumen;

	//Manual Contour Calc
	std::vector <manualContourControler *> _manContourControlCalc;
	std::vector <manualContourModel *> _mContourModelCalc;
	std::vector <manualViewContour *> _mViewContourCalc;

	//Manual Contour Hypo
	std::vector <manualContourControler *> _manContourControlHypo;
	std::vector <manualContourModel *> _mContourModelHypo;
	std::vector <manualViewContour *> _mViewContourHypo;	



};

#endif //
