#ifndef WXPANELCUTTINGIMAGEDATA_H_
#define WXPANELCUTTINGIMAGEDATA_H_

#include "vtkClipping3DDataViewer.h"

#include "figureCuttingCylinderModel.h"
#include "figureCuttingCubeModel.h"
#include "figureCuttingSphereModel.h"
#include "figureCuttingModel.h"

#include "wxVtk3DBaseView.h"

#include "vtkXYPlotActor.h"
#include "vtkCubeSource.h"
#include "vtkCylinderSource.h"
#include "vtkSphereSource.h"

#include "vtkPolyDataMapper.h"

class wxPanelCuttingImageData : public wxPanel
{
public:
	wxPanelCuttingImageData (wxWindow *parent);
	~wxPanelCuttingImageData ();
	void OnTransform(wxScrollEvent& event);
	void OnOpacityFig(wxScrollEvent& event);
	void OnTypeFig(wxCommandEvent& event);
	void OnExtract(wxCommandEvent& event);
	void SetVtkMPRBaseData( vtkMPRBaseData *vtkmprbasedata );
	void SetVtkClipping3DDataViewer( vtkClipping3DDataViewer *vtkclipping3Ddataviewer );
	void SetWxVtk3DBaseView( wxVtk3DBaseView * wxvtk3Dbaseview );

	void SetParamsOfTransformation( );
	void Configure();
	void Refresh();
	void RefreshView();
	void RemoveActors();


private:
	wxSlider						*_opacityFig;
	wxSlider						*_scaleX;
	wxSlider						*_scaleY;
	wxSlider						*_scaleZ;
	wxSlider						*_rotationX;
	wxSlider						*_rotationY;
	wxSlider						*_rotationZ;
	
	wxRadioButton					*_volIntern;
	wxRadioButton					*_volExtern;
	wxCheckBox						*_histogrammeAccumulated;
	wxSlider						*_isoValue;
	wxSlider						*_valueBeforeIsoValue;
	wxSlider						*_valueAfterIsoValue;
	wxChoice						*_typeFig;

	wxStaticText					*_infoToVo;
	wxStaticText					*_infoSuVo;
	wxStaticText					*_infoSuVoA;
	wxStaticText					*_infoPixLe;
	wxStaticText					*_infoPixHi;

	// Model
	figureCuttingCylinderModel		*_modelCylinder;
	figureCuttingCubeModel			*_modelCube;
	figureCuttingSphereModel		*_modelSphere;
	figureCuttingModel				*_actualCuttingModel;

	// view
	vtkCubeSource					*_vtkcube;
	vtkSphereSource					*_vtksphere;
	vtkCylinderSource				*_vtkcylinder;
	vtkPolyDataMapper				*_cubeMapper;
	vtkPolyDataMapper				*_sphereMapper;
	vtkPolyDataMapper				*_cylinderMapper;
	vtkActor						*_cubeActor;
	vtkActor						*_sphereActor;
	vtkActor						*_cylinderActor;
	vtkActor						*_actualActor;


	vtkMPRBaseData					*_vtkmprbasedata;
	wxVtk3DBaseView					*_wxvtk3Dbaseview;
	vtkImageData					*_imageData;
	vtkClipping3DDataViewer			*_vtkclipping3Ddataviewer;

	vtkImageData					*_histogrammeVector;
	vtkXYPlotActor					*_xyplot;
	wxVtkBaseView					*_wxvtkbaseView;
	vtkRenderer						*_renplotter;

	void	CreateModel();
	void	CreateInterface();
	void	Create3DViewObjects();
	void	RefreshOpacity();
	wxWindow *CreatePlotHistogrammeInterface();
	void	InitHistogramme();


protected:
};

#endif /*WXPANELCUTTINGIMAGEDATA_H_*/
